(* Deux classes du mme type *)

class counter (initial) = object
  val mutable c = initial
  method increment () = c <- c+1
  method show () = string_of_int c
end

class counter2 (initial1, initial2) = object
  val mutable c1 = initial1
  val mutable c2 = initial2
  method increment () = c1 <- c1 + 1 ; c2 <- c2 * c2
  method show () = "[" ^ (string_of_int c1) ^ " , " ^ (string_of_int c2) ^ "]" 
end

let x = new counter 0;;
x#increment ();;
x#show ();;
let y = new counter2 (1,2);;
y#increment ();;
y#show ();;
x=y;; (* x et y sont du mme type ! *)

(* les noms des classes sont simplement des abbrviations pour le type *)
(x:counter) = (y:counter2);;

(* Dfinition d'une classe de type diffrent *)

class counter3 (initial) = object
  val mutable c = initial
  method incrementer () = c <- c+1
  method montrer () = string_of_int c
end

let z = new counter3 0;;

x = z ;; (* les types de x et z sont diffrents *)

(* Quand est-ce que deux objets sont gaux *)

let x1 = new counter 0;;
let x2 = new counter 0;;

x1 = x2 ;;
x1 = x1;;

(* chaque objet  sa propre identit *)




(* Simulation partielle en CAML sans objets. *)

type counter_type = {
  increment: unit -> unit;
  show: unit -> string
}

let create_counter (initial) =
  let c = ref initial
  in
    { 
      increment = (function () -> c := !c+1);
      show = (function () -> string_of_int !c)
    };;

let create_counter2 (initial1, initial2) =
  let c1 = ref initial1
  and c2 = ref initial2
  in
    {
      increment = (function () -> c1 := !c1+1; c2 := !c2*(!c2));
      show = (function () ->
		"[" ^ (string_of_int !c1) ^ " , " ^ (string_of_int !c2) ^ "]")
    };;

(* les deux fonctions de cration renvoient le mme type *)
create_counter;;
create_counter2;;

let x = create_counter 0;;
x.increment ();;
x.show ();;
let y = create_counter2 (1,2);;
y.increment ();;
y.show ();;

x = x;;

(* cela ne marche pas parceque OCaml ne peut pas comparer les fonctions *)
x = y;; 

(* L'identit des valeurs de type  enregistrement de fonctions  est 
   diffrent de l'identit d'objets.
*)
