(* Une classe trs simple : les compteurs. On peut la simuler parfaitement
   dans CAML sans les objets.
*)

class counter (initial) = object
  val mutable c = initial
  method increment () = c <- c+1
  method show () = c
end

(* Maintenant on peut crer deux compteurs indpendants *)
let x = new counter 0;;
x#increment ();;
x#increment ();;
x#show ();;
let y = new counter 42;;
y#increment ();;
y#show ();;
x#show ();;

(* les variables d'instaces ne sont pas visibles *)
x#c;; (* erreur *)


(* Simulation en CAML sans objets. La variable d'instance x de la classe est
   ici simule par un identificateur local qui n'est visible que dans les
   cltures des fonctions increment et show.
*)

type counter = {
  increment: unit -> unit;
  show: unit -> int
}

let create_counter (initial) =
  let c = ref initial
  in
    { 
      increment = (function () -> c := !c+1);
      show = (function () -> !c)
    }

let x = create_counter 0;;
x.increment ();;
x.increment ();;
x.show ();;
let y = create_counter 42;;
y.increment ();;
y.show ();;
x.show ();;

(* l'identificateur c n'est pas un champ des enregistrements du type counter *)
x.c;; (* erreur *)
