(** Price policies with pattern matching *)

class virtual equipment = 
  object 
    method virtual name : unit -> string
    method virtual price : unit -> int
  end

class floppy () =
  object 
    inherit equipment
    method name () = "floppy"
    method price1 () = 10
    method price () = 5
  end 


class chassis () = 
  object 
    inherit equipment
    method name () = "chassis" 
    method price2 () = 15
    method price1 () = 12
    method price () = 10
  end 

class virtual equipmentvisitor = 
  object 
    method virtual visitFloppy : floppy -> unit 
    method virtual visitChassis : chassis -> unit 
    method virtual gettotal : unit 
  end 


class pricevisitor () = 
  object 
    inherit equipmentvisitor
    val mutable totalprice = 0
    method visitFloppy f = totalprice <- totalprice + f#price1 ()
    method visitChassis c = totalprice <- totalprice + c#price2 ()
    method gettotal = print_int totalprice
  end 

type equip = Floppy of floppy | Chassis of chassis

let visit e (v:equipmentvisitor) = 
  (* in Java, this code would be a bunch of i-t-e and instaceOf *)
  match e with 
    (Floppy e1) -> v#visitFloppy e1 
  | (Chassis e2) -> v#visitChassis e2


(* use of the classes *) 

let a = new floppy ();;
let b = new chassis ();; 

let equips = [Floppy a ; Chassis b];;

let v = new pricevisitor();;
List.iter (fun e -> visit e v) equips;;

v#gettotal;;







(* If we go this way, we can be much more concise: *)

let visitprice e tot = 
  match e with 
    (Floppy e1)  -> tot + e1#price1 ()
  | (Chassis e2) -> tot + e2#price2 ()
;;

List.fold_right visitprice equips 0;;


